from horsebox.collectors.guessers.html import guess_html  # noqa: F401
from horsebox.collectors.guessers.raw import guess_raw  # noqa: F401
from horsebox.collectors.guessers.rss import guess_rss  # noqa: F401
